/* This file is part of Cloudy and is copyright (C)1978-2006 by Gary J. Ferland
 * For conditions of distribution and use see copyright notice in license.txt */
/*DR_Badnell_init This code is written by Terry Yun, 2005 *
 * It reads rate coefficient fits into 3D arrays and output array.out for testing *
 * The testing can be commented out */
/*DR_Badnell_rate This code is written by Terry Yun, 2005 *
 * It interpolates the rate coefficients in a given temperature.*
   It recieves ATOMIC_NUM_BIG, NELECTRONS values, temperature and returns the rate coefficient*
   It returns
        '-2': initial <= final
              init < 0 or init >302 or final < 0 or final > 302
        '-1': the transition is not defined
        '99': unknown invalid entries                         */ 
#include "cddefines.h"
#include "phycon.h"
#include "path.h"
#include "ionbal.h"

#define SIZE_OF_LINE 300
#define MAX_FIT_PAR 9   
/* one larger than largest atomic number in Nigel's data file */
#define ATOMIC_NUM_BIG 55
/* number of electrons */
#define NELECTRONS 12
static double C[ATOMIC_NUM_BIG][NELECTRONS][MAX_FIT_PAR]; 
static double E[ATOMIC_NUM_BIG][NELECTRONS][MAX_FIT_PAR];    
static int defn_C[ATOMIC_NUM_BIG][NELECTRONS];   
static int defn_E[ATOMIC_NUM_BIG][NELECTRONS];    
static int nn_counter[ATOMIC_NUM_BIG][NELECTRONS]; 

/*#define PRINT */ 
#ifdef	PRINT
#define FILE_NAME_OUT "array.out"    
#endif
   
/*DR_Badnell_init This code is written by Terry Yun, 2005 *
 * It reads rate coefficient fits into 3D arrays and output array.out for testing *
 * The testing can be commented out */
void DR_Badnell_init( void )
{

	double par_C[MAX_FIT_PAR];
	double par_E[MAX_FIT_PAR];
	char line[SIZE_OF_LINE];
	int z_value, n_value;
	int i, j, k;
	int count, number;
#	define NBLOCK	2
	int data_begin_line[NBLOCK];/*it tells you where the data set begins(begins with 'Z')*/
	int length_of_line;	/*this variable checks for a blank line*/
	FILE *ioDATA;
	char chFilename[FILENAME_PATH_LENGTH_2] ;

#	ifdef DEBUG_FUN
	fputs( "<+>DR_Badnell_init()\n", debug_fp );
#	endif

	for( i=0; i<NBLOCK; ++i )
	{
		/* set to really large negative number - crash if used before being redefined */
		data_begin_line[i] = INT_MIN;
	}

	/* check on path is file not here and path set */
	/* path was parsed in getset */
	if( lgDataPathSet == TRUE )
	{
		/*path set, so look only there */
		strcpy( chFilename , chDataPath );
		strcat( chFilename , "badnell_dr.dat" );
	}
	else
	{
		/* path not set, check local space only */
		strcpy( chFilename , "badnell_dr.dat" );
	}

	if( ( ioDATA = fopen( chFilename , "r" ) ) == NULL )
	{
		printf("DR_Badnell_init cannot open %s\n", chFilename);
		exit(1);
	}

	#ifdef PRINT
	ofp= fopen(FILE_NAME_OUT, "w");
	if( ofp == NULL )
	{
		printf("Can't open %s\n", FILE_NAME_OUT);
		exit(1);
	}
	#endif
   
	count = 0;
	number = 0;

	/*Find out the number line where the data starts 
	 * there are two main blocks of data and each starts with a Z in column 2 */
	while( fgets(line, (int)sizeof(line), ioDATA) != NULL )
	{
		count++;

		if( line[2]=='Z' )
		{
			/* number has to be 0 or 1, and indicates the first or second block of data
			 * count is the line number for the start of that block */
			data_begin_line[number] = count;
			ASSERT( number < NBLOCK );
			number++;
		}
	}

	/*set a flag for a undefined data*/
	for( i=0; i<ATOMIC_NUM_BIG; i++ )
	{
		for( j=0; j<NELECTRONS; j++ )
		{
			defn_C[i][j] = 0;
			defn_E[i][j] = 0;   
		}
	}

	/*set fitting coeffs to zero initially*/
	for( i=0; i<ATOMIC_NUM_BIG; i++ )
	{
		for( j=0; j<NELECTRONS; j++ )
		{
			for( k=0; k<MAX_FIT_PAR; k++ )
			{
				C[i][j][k] = 0;
				E[i][j][k] = 0;  
			}
		}
	}

	count = 0;

	/*Start from beginning to read in again*/  
	fseek(ioDATA, 0, SEEK_SET);
	while( fgets(line, (int)sizeof(line), ioDATA) != NULL )
	{
		count++;
		length_of_line = (int)strlen(line);

		/*reading in coeffs C */
		if( count > data_begin_line[0] && count < data_begin_line[1] && length_of_line >3 )
		{
			/*set array par_C to zero */
			for( i=0; i<MAX_FIT_PAR; i++ )
			{
				par_C[i] = 0;
			}
			sscanf(line, "%i%i%lf%lf%lf%lf%lf%lf%lf%lf%lf", &z_value, &n_value, &par_C[0], &par_C[1], &par_C[2], &par_C[3], &par_C[4], &par_C[5], &par_C[6], &par_C[7], &par_C[8]);

			/*Set a flag to '1' when the indices are defined */  
			defn_C[z_value][n_value] = 1;

			/*counting the number of coeffs */
			nn_counter[z_value][n_value] = 8;   
			while( par_C[nn_counter[z_value][n_value]] == 0 )
			{
				--nn_counter[z_value][n_value];
			}
			++nn_counter[z_value][n_value];

			/*assign the values into array */
			for( i=0; i<nn_counter[z_value][n_value]; i++ )
				C[z_value][n_value][i] = par_C[i];
		}
	}

	/*starting again to read in E values */
	fseek(ioDATA, 0, SEEK_SET);  
	count = 0; 
	while( fgets(line, (int)sizeof(line), ioDATA) != NULL )
	{
		count++;
		length_of_line = (int)strlen(line);
		if( count > data_begin_line[1] && length_of_line > 3 )
		{

		  /*set array par_E to zero*/
			for( i=0; i<MAX_FIT_PAR; i++ )
			{
				par_E[i] = 0;
			}   
			sscanf(line, "%i%i%lf%lf%lf%lf%lf%lf%lf%lf%lf", &z_value, &n_value, &par_E[0], &par_E[1], &par_E[2], &par_E[3], &par_E[4], &par_E[5], &par_E[6], &par_E[7], &par_E[8]);


			/*counting the number of coeffs*/
			nn_counter[z_value][n_value] = 8;   
			while( par_E[nn_counter[z_value][n_value]] == 0 )
			{
				--nn_counter[z_value][n_value];
			}
			++nn_counter[z_value][n_value];

			/*Set a flag to '1' when the indices are defined*/   
			defn_E[z_value][n_value] = 1;

			/*assign the values into array*/
			for( i=0; i<nn_counter[z_value][n_value]; i++ )
				E[z_value][n_value][i] = par_E[i];   
		}
	} 

	/*output coeffs for defined values for testing */
	#ifdef PRINT
	for( i=0; i<ATOMIC_NUM_BIG; i++ )
	{
		for( j=0; j<NELECTRONS; j++ )
		{
			if( defn_C[i][j] == 1 )
			{
				fprintf(ofp, "%i %i %e %e %e %e %e %e %e %e %e\n", i, j, C[i][j][0], C[i][j][1], C[i][j][2], C[i][j][3], C[i][j][4], C[i][j][5], C[i][j][6], C[i][j][7], C[i][j][8]);
			}
		}
	}
	for( i=0; i<ATOMIC_NUM_BIG; i++ )
	{
		for( j=0; j<NELECTRONS; j++ )
		{
			if( defn_E[i][j] == 1 )
			{
				fprintf(ofp, "%i %i %e %e %e %e %e %e %e %e %e\n", i, j, E[i][j][0], E[i][j][1], E[i][j][2], E[i][j][3], E[i][j][4], E[i][j][5], E[i][j][6], E[i][j][7], E[i][j][8]);
			}
		}
	}
   	fclose(ofp);
   #endif
   
	/*checking for the match of defn_C and defn_E - Both have to be defined*/
	for( i=0; i<ATOMIC_NUM_BIG; i++ )
	{
		for( j=0; j<NELECTRONS; j++ )
		{
			if( defn_C[i][j] != defn_E[i][j] )
			{
				printf("C%i, E%i: %i %i\n", i, j, defn_C[i][j], defn_E[i][j]);
				printf("Critical error in defn\n");
			}
		}
	}

	fclose(ioDATA);

#	ifdef DEBUG_FUN
	fputs( " <->DR_Badnell_init()\n", debug_fp );
#	endif

	return;
}

/*DR_Badnell_rate This code is written by Terry Yun, 2005 *
 * It interpolates the rate coefficients in a given temperature.*
   It recieves ATOMIC_NUM_BIG, NELECTRONS values, temperature and returns the rate coefficient*
   It returns
        '-2': initial <= final
              init < 0 or init >302 or final < 0 or final > 302
        '-1': the transition is not defined
        '99': unknown invalid entries                         */ 
double DR_Badnell_rate(
	/* atomic number on physics scale - He - 2 */
	int z_val, 
	/* number of core electrons before capture of free electron */
	int n_val )
{

	double k, sum;
	int i;

#	ifdef DEBUG_FUN
	fputs( "<+>DR_Badnell_rate()\n", debug_fp );
#	endif

	/*Invalid entries returns '-2':if the z_values are smaller than equal to the n_values */
	if( z_val <= n_val )
	{
		k = -2;
	}
	/*Invalid intries returns '-2' if z_val and n_val are out of the range*/
	else if( z_val <2 || z_val > 30 || n_val < 1 || n_val > 11 )
	{
		k = -2;
	}
	/*undefined z and n returns '-1'*/
	else if( defn_C[z_val][n_val] == 0 )
	{
		k = -1;
	}
	else if( defn_C[z_val][n_val] == 1 )
	{
		sum = 0;
		i = 0;
		/* loop over all non-zero terms */
		for(i=0; i<nn_counter[z_val][n_val]; ++i  )
		{
			sum += (C[z_val][n_val][i] * sexp( E[z_val][n_val][i]/phycon.te));
		}

		/*k = pow(phycon.te, -1.5) * sum;*/
		k = sum / phycon.te32;
	}
	/*unkown invalid entries returns '-99'*/
	else
	{
		k = -99;
	}

#	ifdef DEBUG_FUN
	fputs( " <->DR_Badnell_rate()\n", debug_fp );
#	endif

	return k;

}

